#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
import csv
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import datetime
from scipy.ndimage.filters import gaussian_filter1d
#matplotlib.use('Agg')
    
# Constants from SR104 resistor lid
ALPHA    = 0.089  # Alpha value at +23c
BETA     = -0.0031 # Beta value
R_DEV    = 0    # Deviation value for standard resistor
TEMP_DEV = 0.004  # Deviation value for RT
STEP     = 0.5    # Step to go thru temperature points in mode 2
STDREF   = 9998.0
#count = 0
fn_input = "test5_732bx_ramon_mx_k2400psu_jun2019.dsv"


def show_r(temp, rsi):
    f = 0
    delta = temp - 23.0
    ppm = 0.0 + (ALPHA * delta) + (BETA * delta * delta)
    r = STDREF + ((ppm+R_DEV) * STDREF / 1000000.0)
    f = (temp * 9.0 / 5.0) + 32.0
    print ("temp = %5.2fC / %6.2fF ppm = %7.4f Rstd = %12.8f Ohms, %3.3f ppm\n" % (temp,f, R_DEV+ppm, r, (((r / rsi) - 1) *1e6 ) ) ),
    return r

def calc_ppmdev(temp, rsi, a, b):
    f = 0
    rdev = 0
    delta = temp - 23.0
    ppm = 0.0 + (a * delta) + (b * delta * delta)
    r = rsi + ((ppm+rdev) * rsi / 1000000.0)
    f = (temp * 9.0 / 5.0) + 32.0
    #print ("temp = %5.2fC / %6.2fF ppm = %7.4f Rstd = %12.8f Ohms, %3.3f ppm\n" % (temp,f, R_DEV+ppm, r, (((r / rsi) - 1) *1e6 ) ) ),
    return (((r / rsi) - 1) *1e6 )
    
print "\033[37;1m-- xDevs.com/EEVBlog RComp | SR104 correction calculator -- \n-- https://xdevs.com/review/sr104/  Rev.2 May 2019           --"
print "R deviation = %.4f ppm    T deviation = %.4f%%\n" % (R_DEV, TEMP_DEV),
print "Alpha = %.4f ppm/C       Beta = %.4f ppm/C/C\n" % (ALPHA, BETA)

#mode = int(raw_input("Enter 1 : calculate Rstd at Rrtd \r\nEnter 2 : table curve generate\r\nEnter 3 : Eat DSV-file to calculate alpha/beta/delta data\r\nEnter 4 : matplotlib sample data\r\nEnter 5 : Eat DCV and plot diag graph\r\n"))
mode = 13
if (mode == 1):
    rt = float(raw_input("Room Temp: "))
    rs = float(raw_input("Standard resistance: "))
    rs = STDREF + (R_DEV * STDREF / 1000000.0)
    print "Rt=%f  Rs=%f\n" % (rt, rs)
    r = (rt - rs) * 100.0 / STDREF
    r = r - TEMP_DEV
    r = r * 10.0
    temp = r + 23.0
    show_r(temp, STDREF)
elif (mode == 12):
    # DSV data parser to calculate alpha/beta values
    global fn_input
    fn_input = "steve_test_august2019.dsv"
    fn_output = "tcr_math_output_rhb.txt"

    temp = []
    val = [[]]
    ral = [[]]
    polyz = []
    uncerr = [[]]
    for ival in range (0,17):
        val.append([])
    for ival in range (0,17):
        ral.append([])
        uncerr.append([])
    #val2 = []
    #val3 = []
    #val4 = []
    #val5 = []
    #val6 = []
    cnt = 0 
    alpha = []
    beta = []
    amb = []
    gain = []
    tvalue = []
    c1 = []
    c2 = []
    delta = []
    zerotctemp = []
    valzerotc = []
    cnx = []
    date = []
    chop_samples = 1
    end_samples = 8300
    
    
    num_lines = sum(1 for line in open(fn_input))
    for dval in range (0,end_samples-chop_samples):
        date.append(" ")
    #vref = [7.151292766e+00,9.991393197e+00,7.158675987e+00,9.983004444e+00,7.136260692e+00,9.962210853e+00,7.151827323e+00,9.981525961e+00,7.108384358e+00,9.923200475e+00,7.156064144e+00,9.999977018e+00,7.160140665e+00,1.000038978e+01,7.162542902e+00,9.996671352e+00]
    #vref = [10.000015240,7.085454509,1.000001524e+01,1.000001524e+01,1.000001524e+01,7.085451500e+00,1.000001524e+01,9999.9995,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,1.200000000e+01]
    vref = [10,10,10,10,10,1.0000,10.00001524,10.00001524,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,1.200000000e+01]
    col_active = 5

    with open(fn_input) as csvfile:
        spamreader = csv.reader(csvfile, delimiter=';', quotechar='|')
        is_first = True
        dcount = 0
        for row in spamreader:
            cnt = cnt + 1
            dcount = dcount + 1
            if is_first:
                is_first = False
                continue
            #print ("%d" % cnt)
            if cnt <= chop_samples:
                continue
            if cnt > end_samples:
                continue
            else:
                cnx.append(cnt - (chop_samples+1))
                temp.extend([ float(row[13]) - 1 ])
                amb.extend([ float(row[13]) ])
                date[dcount - (chop_samples+1)] = datetime.datetime.strptime (row[0],"%d/%m/%Y-%H:%M:%S")
                #date.extend(row[0])#.strftime('%d/%m/%y-%h:%m:%s')
                val[0].extend([ ( (float(row[1]) / vref[0]) - 1) * 1e6 ]) #val1
                val[1].extend([ ( (float(row[2]) / vref[1]) - 1) * 1e6 ]) #val2
                val[2].extend([ ( (float(row[5]) / vref[2]) - 1) * 1e6 ]) #val3
                val[3].extend([ ( (float(row[6]) / vref[3]) - 1) * 1e6 ]) #val4
                val[4].extend([ ( (float(row[7]) / vref[4]) - 1) * 1e6 ]) #val4
                #val[4].extend([ ( (float(row[5]) / vref[4]) - 1) * 1e6 ]) #val5
                #val[5].extend([ ( (float(row[6]) / vref[5]) - 1) * 1e6 ]) #val6
                #val[6].extend([ ( (float(row[7]) / vref[6]) - 1) * 1e6 ]) #val6
                #val[7].extend([ ( (float(row[8]) / vref[7]) - 1) * 1e6 ]) #val6
                uncerr[0].extend([0.2])
                uncerr[1].extend([0.62])
                #
                #val[8].extend([ ( (float(row[9]) / vref[8]) - 1) * 1e6 ]) #val6
                #val[9].extend([ ( (float(row[10]) / vref[9]) - 1) * 1e6 ]) #val6
                #val[10].extend([ ( (float(row[11]) / vref[10]) - 1) * 1e6 ]) #val6
                #val[11].extend([ ( (float(row[12]) / vref[11]) - 1) * 1e6 ]) #val6
                #
                #val[12].extend([ ( (float(row[13]) / vref[12]) - 1) * 1e6 ]) #val6
                #val[13].extend([ ( (float(row[14]) / vref[13]) - 1) * 1e6 ]) #val6
                #val[14].extend([ ( (float(row[15]) / vref[14]) - 1) * 1e6 ]) #val6
                #val[15].extend([ ( (float(row[16]) / vref[15]) - 1) * 1e6 ]) #val6
                
                ral[0].extend([  float(row[1]) ]) #val1
                ral[1].extend([  float(row[2]) ]) #val2
                ral[2].extend([  float(row[5]) ]) #val3
                ral[3].extend([  float(row[6]) ]) #val4
                ral[4].extend([  float(row[7]) ]) #val5
                #ral[5].extend([  float(row[6]) ]) #val6
                #ral[6].extend([  float(row[7]) ]) #val6
                #ral[7].extend([  float(row[8]) ]) #val6
                #ral[8].extend([  float(row[9]) ]) #val6
                #ral[9].extend([  float(row[10]) ]) #val6
                #ral[10].extend([ float(row[11]) ]) #val6
                #ral[11].extend([ float(row[12]) ]) #val6
                #ral[12].extend([ float(row[13]) ]) #val6
                #ral[13].extend([ float(row[14]) ]) #val6
                #ral[14].extend([ float(row[15]) ]) #val6
                #ral[15].extend([ float(row[16]) ]) #val6
                
                #val[24].extend([0.0]) #val8
            #corr.extend([(float(row[2]) + float(row[6]) ) / 2])
            #date.extend([row[0]])
    csvfile.close()
    
    #print amb
    tdelta = np.amax(temp) - np.amin(temp)
    reftemp = 23.00
    nominal = []
    median = []
   
    #print "Temperature min = %.3f , max = %.3f, delta = %.3f C" % (np.amin(temp), np.amax(temp), tdelta)
    #print "REF temp = %.3f C" % reftemp
        
    print "\033[2J"
    with open(fn_output, 'a') as logfile:
        logfile.write ("\r\rFile = %s\r" % fn_input)
    logfile.close()
    
    for zval in range (0,col_active):
        #nominal = [9.999957591e+00, 9.999963348e+00, 0, 0, 1.000001420e+01, 9.999958800e+00, 9.515830000e-01, 0, 0]
        nominal.append(np.mean(ral[zval][:30]))
        polyz.append(np.polyfit(temp, ral[zval], 2))
        c2.append(polyz[zval][0])
        c1.append(polyz[zval][1])
        gain.append(polyz[zval][2])
        tvalue.append(c2[zval] * reftemp*reftemp + c1[zval]*reftemp + gain[zval])
        alpha.append((c1[zval] + 2*reftemp*c2[zval]) / tvalue[zval]*1e6 )
        beta.append(c2[zval] / tvalue[zval]*1e6 )
        delta.append((tvalue[zval] - nominal[zval]) / nominal[zval] * 1e6)
        zerotctemp.append(-alpha[zval] / 2 / beta[zval] + reftemp)
        valzerotc.append(nominal[zval] + (zerotctemp[zval]) * c1[zval] + (zerotctemp[zval])*2*c2[zval])
        median.append(np.median(ral[zval]))
        
        if (gain[zval] != 0):
            print "\033[37;1m\r\nNominal       :", nominal[zval]
            print ("Median        : %.8f" % median[zval])
            print ("Column number : %d " % zval), polyz[zval], "Total samples = %d" % len(ral[zval])
            print ("c1            : %e" % c1[zval])
            print ("c2            : %e" % c2[zval])
            print ("g             : %e" % gain[zval])
            print ("Value T23     : %.6f" % (tvalue[zval]))
            print ("\033[36;1mAlpha T23     : %.4f ppm/K       " % (alpha[zval])),
            print ("Beta      : %.4f ppm/K/2                " % (beta[zval])),
            print ("Delta T23 : %.3f ppm         \033[39;1m" % (delta[zval]))
            print ("\033[35;1mZero TC       : %.3f C \033[39;0m" % (zerotctemp[zval])),
            print ("\033[32;1m          Delta ZeroTC : %.6f\033[39;0m" % (valzerotc[zval]))
            
            with open(fn_output, 'a') as logfile:
                logfile.write ("\rColumn number : %d\r" % zval), polyz[zval], "Total samples = %d" % len(ral[zval])
                logfile.write ("Nominal       : %.8f\r" % nominal[zval])
                logfile.write ("c1            : %e\r" % c1[zval])
                logfile.write ("c2            : %e\r" % c2[zval])
                logfile.write ("g             : %e\r" % gain[zval])
                logfile.write ("Value T23     : %.8f\r" % (tvalue[zval]))
                logfile.write ("Alpha T23     : %.4f ppm/K     \r" % (alpha[zval])),
                logfile.write ("Beta          : %.4f ppm/K/2   \r" % (beta[zval])),
                logfile.write ("Delta T23     : %.3f ppm       \r" % (delta[zval]))
                logfile.write ("Zero TC       : %.3f C \r" % (zerotctemp[zval])),
                logfile.write ("Delta ZeroTC  : %.8f   \r" % (valzerotc[zval]))
            logfile.close()
    w = 20
    h = 16
    fig,ax = plt.subplots()
    fig.set_size_inches(w,h)
    plt.tight_layout(rect=[0.03, 0.03, 0.96, 0.98], w_pad=0.4, h_pad=0.4)                                # Reduce margins around the graph
        
    plt.xlabel('Timescale', fontsize=18)
    plt.ylabel('Deviation from 10V, ppm', fontsize=18) #$\mu$V
    
    #new_x = dates.num2date(date)
    avgpower = 16
    
    val1aver = gaussian_filter1d(val[0], sigma=avgpower)
    val2aver = gaussian_filter1d(val[1], sigma=avgpower)
    val3aver = gaussian_filter1d(val[2], sigma=avgpower)
    val4aver = gaussian_filter1d(val[3], sigma=avgpower)
    val5aver = gaussian_filter1d(val[4], sigma=avgpower)
    #val6aver = gaussian_filter1d(val[5], sigma=avgpower)
    #val7aver = gaussian_filter1d(val[6], sigma=avgpower)
    #val8aver = gaussian_filter1d(val[7], sigma=avgpower)
    #
    #val9aver = gaussian_filter1d(val[8],   sigma=avgpower)
    #val10aver = gaussian_filter1d(val[9],  sigma=avgpower)
    #val11aver = gaussian_filter1d(val[10], sigma=avgpower)
    #val12aver = gaussian_filter1d(val[11], sigma=avgpower)
    #val13aver = gaussian_filter1d(val[12], sigma=avgpower)
    #val14aver = gaussian_filter1d(val[13], sigma=avgpower)
    #val15aver = gaussian_filter1d(val[14], sigma=avgpower)
    #val16aver = gaussian_filter1d(val[15], sigma=avgpower)
    
    temp2aver = gaussian_filter1d(temp, sigma=1)
    
    colors = ['#00aa00', '#005599', '#009999', '#551ac1', '#e51ac1', '#f58231', '#ff0000', '#f032e6', '#00a68c', '#664433', '#1f38e3', '#ff0000', '#ec18a0', '#dbb201', '#1c2b33', '#003355', '#fc007f']
    
    avglinestype = 'none'
    avglinestypen = 'none'
    
    plt.title('TheSteve snake 5 DMMs, 10VDC log from 732B', fontsize=20)
    ax.plot(temp, val1aver, '-', label='CHA',  linewidth=1.0, color=colors[0], linestyle=avglinestype, marker='.', ms=2)
    ax.plot(temp, val2aver, '-', label='CH2',  linewidth=1.0, color=colors[1], linestyle=avglinestype, marker='.', ms=2)
    ax.plot(temp, val3aver, '-', label='CH3',  linewidth=1.0, color=colors[2], linestyle=avglinestype, marker='.', ms=2)
    ax.plot(temp, val4aver, '-', label='CHB',  linewidth=1.0, color=colors[3], linestyle=avglinestype, marker='.', ms=2)
    ax.plot(temp, val5aver, '-', label='CHC',  linewidth=1.0, color=colors[4], linestyle=avglinestype, marker='.', ms=2)
    #ax.plot(date, val6aver, '-', label='CH6',  linewidth=2.0, color=colors[5], linestyle=avglinestype, marker='.', ms=5)
    #ax.plot(date, val7aver, '-', label='CH7',  linewidth=2.0, color=colors[6], linestyle=avglinestype, marker='.', ms=5)
    #ax.plot(date, val8aver, '-', label='CH8',  linewidth=2.0, color=colors[7], linestyle=avglinestype, marker='.', ms=5)
    #ax.plot(date, val9aver, '-', label='CH1',  linewidth=3.0, color=colors[4], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val10aver, '-', label='CH2', linewidth=1.0, color=colors[4], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val11aver, '-', label='CH3', linewidth=3.0, color=colors[5], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val12aver, '-', label='CH4', linewidth=6.0, color=colors[6], linestyle=avglinestype)#, marker='.', ms=1)
    #ax.plot(date, val13aver, '-', label='CH5', linewidth=3.0, color=colors[7], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val14aver, '-', label='CH6', linewidth=1.0, color=colors[7], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val15aver, '-', label='CH7', linewidth=3.0, color=colors[8], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val16aver, '-', label='CH8', linewidth=1.0, color=colors[8], linestyle=avglinestype, marker='.', ms=1)
    
    yerr = np.linspace(0.05, 0.2, 10)
    #plt.errorbar(date, val1aver, uncerr[0], errorevery = 50,  color=colors[0], uplims=True, lolims=True, linewidth=0.5, mew=0.2, label='both limits (default)')
    #plt.errorbar(date, val2aver, uncerr[0], errorevery = 40,  color=colors[3], uplims=True, lolims=True, linewidth=0.5, mew=0.2, label='both limits (default)')
    #plt.errorbar(date, val8aver, uncerr[1], errorevery = 70,  color=colors[7], uplims=True, lolims=True, linewidth=0.5, mew=0.2, label='both limits (default)')
    
    
    ax.plot(temp, val[0], '-'  , label='CHA',  linewidth=0.8, alpha=0.2, color=colors[0] )#, linestyle='none', marker='.', ms=6)
    ax.plot(temp, val[1], '-'  , label='CH2',  linewidth=0.8, alpha=0.2, color=colors[1] )#, linestyle='none', marker='.', ms=6)
    ax.plot(temp, val[2], '-'  , label='CH3',  linewidth=0.8, alpha=0.2, color=colors[2] )#, linestyle='none', marker='.', ms=6)
    ax.plot(temp, val[3], '-'  , label='CHB',  linewidth=0.8, alpha=0.2, color=colors[3] )#, linestyle='none', marker='.', ms=6)
    ax.plot(temp, val[4], '-'  , label='CH4',  linewidth=0.8, alpha=0.2, color=colors[4] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[5], '-'  , label='CH6',  linewidth=1.0, alpha=0.3, color=colors[5] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[6], '-'  , label='CH7',  linewidth=1.0, alpha=0.3, color=colors[6] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[7], '-'  , label='CH8',  linewidth=1.0, alpha=0.3, color=colors[7] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[8], '-'  , label='CH1',  linewidth=1.0, alpha=0.3, color=colors[4] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[9], '-'  , label='CH2',  linewidth=1.0, alpha=0.3, color=colors[4] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[10], '-'  , label='CH3', linewidth=1.0, alpha=0.3, color=colors[5])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[11], '-'  , label='CH4', linewidth=1.0, alpha=0.3, color=colors[6])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[12], '-'  , label='CH5', linewidth=1.0, alpha=0.3, color=colors[7])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[13], '-'  , label='CH6', linewidth=1.0, alpha=0.3, color=colors[7])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[14], '-'  , label='CH7', linewidth=1.0, alpha=0.3, color=colors[8])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[15], '-'  , label='CH8', linewidth=1.0, alpha=0.3, color=colors[8])#, linestyle='none', marker='.', ms=6)
    
    
    #ax.plot(date, temp2aver, '-', label='TEMP', linewidth=3.0, color='#000000', linestyle='dotted', marker='.', ms=1)
    
    #start, end = ax.get_ylim()
    #startx, endx = ax.get_xlim()
    axis_y_step = 0.01
    #axis_x_step = 1
    #ex = np.arange(1,91000)
    #ax.yaxis.set_ticks(np.arange(start, end)) # axis_y_step))
    ax.yaxis.set_ticks(np.arange(-4.0, 2, 0.1)) # axis_y_step))
    ax.set_ylim(auto=False)
    ax.xaxis.set_ticks(np.arange(20.0, 26, 0.1)) # axis_y_step))
    #ax.xaxis.set_ticks(np.arange(0, 90000, 5000)) # axis_y_step))
    #ax.xaxis.set_ticks(np.arange(startx, endx)) #, axis_x_step))
    
    #ax2 = ax.twinx()  # instantiate a second axes that shares the same x-axis
    #ax2.set_ylabel(u'Temperature, °C', fontsize=18)  # we already handled the x-label with ax1
    ax.margins(x=0)
    #ax2.margins(x=0)
    #ax2.plot(date, temp, '-.', label='TEMP?', linewidth=1.0, color='#ff0000', linestyle='dashed', marker='.', ms=4)
    #ax2.plot(cnx, temp, '-.', label='TEMP?', linewidth=1.0, color='#000000', linestyle='dashed', marker='.', ms=4)
    #ax2.tick_params(axis='y', labelcolor='#000000')
    legendlabel = ["3458[4]", "34461A", "34401A", "D1281", "3458[3]"]#, "3458-10", "3458-11", "2002-6"]#, "3458D vs 732B"]#, "K2002-6 vs 732B", "K2002-4", "8508A vs 732B", "D1281 vs SR104"]# "9.9815132 CH8", "7.1083426 CH9", "9.9231395 CH10", "7.1560244 CH11", "FX 792X", "7.1601550 CH13", "1.0000411 CH14", "7.1625227 CH15", "9.9966437 CH16", "Zero Ref"]
    #legendlabel2 = [u"Ambient, °C"]
    #ax.legend(legendlabel, loc=4, fancybox=True, framealpha=1, borderpad=1, markerscale=4)
    ax.grid(True, color='#cccccc')
    #ax2.legend(legendlabel2, loc=2, fancybox=True, framealpha=1, borderpad=1, markerscale=4)
    #ax2.yaxis.set_ticks(np.arange(18, 26, 0.1)) # axis_y_step))
    #ax.xaxis.set_major_formatter(mdates.DateFormatter("%b %d, %H:%M:%S"))
    #ax.xaxis.set_minor_formatter(mdates.DateFormatter("%b %d, %H:%M:%S"))

    vmed = []
    vmin = []
    vmax = []
    sdev = []
    
    tmin = np.amin(temp)
    tmax = np.amax(temp)
    temppp = tmax - 18
    tempco = []
    colormap = [0,1,2,3,4,5,6,7,8,9,5,6,7,7,8,8]
    
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        #label.set_fontname('Arial')
        label.set_fontsize(13)
    
    for stats in range (0, col_active):
        vmed.extend([0])
        vmin.extend([0])
        vmax.extend([0])
        sdev.extend([0])
        tempco.extend([0])
        
    for stats in range (0, col_active):
    
        if (stats == 5):
            continue
    
        vmed[stats] = np.median(val[stats])
        vmin[stats] = np.amin(val[stats])# - vstep
        vmax[stats] = np.amax(val[stats])# + vstep
        sdev[stats] = np.std(val[stats]) # SDEV
        tempco[stats] = ((vmax[stats] - vmin[stats]) / temppp) 
        if ((stats % 2) == 0):
            bgclr = "#ffffff"
        else:
            bgclr = "#eeeeff"
        
        spacing_xleg = 0.1
        
        yspacing = 0.0175
        
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 1), 'Max[%d] : %.3f ppm' % (stats, vmax[stats]),
            horizontalalignment='left',
            verticalalignment='top', backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 2), 'Min[%d] : %.3f ppm' % (stats, vmin[stats]),
            horizontalalignment='left',
            verticalalignment='top', backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 3),'Median[%d] : %.3f ppm' % (stats, vmed[stats]),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 4),'Sdev[%d] : %.3f ppm' % (stats, sdev[stats]),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        #if (stats >6):
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 10.5), '%s' % (legendlabel[stats - 1]),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=12, color=colors[colormap[stats]])
        #else:
        #    plt.text(0.01 + (stats * spacing_xleg), (yspacing * 6.5), '%s' % (legendlabel[stats]),
        #        horizontalalignment='left',
        #        verticalalignment='top',backgroundcolor=bgclr,
        #        transform = ax.transAxes, size=12, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 9.5), 'R: %.7e' % (vref[stats]),
            horizontalalignment='left',
            verticalalignment='top', backgroundcolor=bgclr,
            transform = ax.transAxes, size=12, color=colors[colormap[stats]])
        
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 5), 'TCbox : %.3f ppm/K' % ( tempco[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 6), 'Alpha : %.3f ppm/K' % ( alpha[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 7), 'Beta : %.3f ppm/K2' % ( beta[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 8), 'Zero TC : %.3f C' % ( zerotctemp[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        
    fig.patch.set_facecolor('#eeeeff')
    smax = len(val[0])
    plt.text(0.2, -0.04,'xDevs.com | https://xdevs.com/article/teckit',
        horizontalalignment='center',
        verticalalignment='center',
        transform = ax.transAxes, size=12)
    plt.text(-0.01, -0.03,'Samples : %d' % smax,
        horizontalalignment='left',
        verticalalignment='top',
        transform = ax.transAxes, size=12)
    
    #plt.ylim(-2,2)
    plot_out_filename = "test_precal-scan_test_steve.png"
    fig.savefig(plot_out_filename, facecolor=fig.get_facecolor(), transparent=False)
    plt.show()
elif (mode == 13):
    # DSV data parser to calculate alpha/beta values
    global fn_input
    fn_input = "steve_test_august2019.dsv"
    fn_output = "tcr_math_output_rhb.txt"

    temp = []
    val = [[]]
    ral = [[]]
    polyz = []
    uncerr = [[]]
    for ival in range (0,17):
        val.append([])
    for ival in range (0,17):
        ral.append([])
        uncerr.append([])
    #val2 = []
    #val3 = []
    #val4 = []
    #val5 = []
    #val6 = []
    cnt = 0 
    alpha = []
    beta = []
    amb = []
    gain = []
    tvalue = []
    c1 = []
    c2 = []
    delta = []
    zerotctemp = []
    valzerotc = []
    cnx = []
    date = []
    chop_samples = 1
    end_samples = 8300
    
    
    num_lines = sum(1 for line in open(fn_input))
    for dval in range (0,end_samples-chop_samples):
        date.append(" ")
    #vref = [7.151292766e+00,9.991393197e+00,7.158675987e+00,9.983004444e+00,7.136260692e+00,9.962210853e+00,7.151827323e+00,9.981525961e+00,7.108384358e+00,9.923200475e+00,7.156064144e+00,9.999977018e+00,7.160140665e+00,1.000038978e+01,7.162542902e+00,9.996671352e+00]
    #vref = [10.000015240,7.085454509,1.000001524e+01,1.000001524e+01,1.000001524e+01,7.085451500e+00,1.000001524e+01,9999.9995,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,1.200000000e+01]
    vref = [10,10,10,10,10,1.0000,10.00001524,10.00001524,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,0.000000000e+00,1.200000000e+01]
    col_active = 5

    with open(fn_input) as csvfile:
        spamreader = csv.reader(csvfile, delimiter=';', quotechar='|')
        is_first = True
        dcount = 0
        for row in spamreader:
            cnt = cnt + 1
            dcount = dcount + 1
            if is_first:
                is_first = False
                continue
            #print ("%d" % cnt)
            if cnt <= chop_samples:
                continue
            if cnt > end_samples:
                continue
            else:
                cnx.append(cnt - (chop_samples+1))
                temp.extend([ float(row[13]) - 1 ])
                amb.extend([ float(row[13]) ])
                date[dcount - (chop_samples+1)] = datetime.datetime.strptime (row[0],"%d/%m/%Y-%H:%M:%S")
                #date.extend(row[0])#.strftime('%d/%m/%y-%h:%m:%s')
                val[0].extend([ ( (float(row[1]) / vref[0]) - 1) * 1e6 ]) #val1
                val[1].extend([ ( (float(row[2]) / vref[1]) - 1) * 1e6 ]) #val2
                val[2].extend([ ( (float(row[5]) / vref[2]) - 1) * 1e6 ]) #val3
                val[3].extend([ ( (float(row[6]) / vref[3]) - 1) * 1e6 ]) #val4
                val[4].extend([ ( (float(row[7]) / vref[4]) - 1) * 1e6 ]) #val4
                #val[4].extend([ ( (float(row[5]) / vref[4]) - 1) * 1e6 ]) #val5
                #val[5].extend([ ( (float(row[6]) / vref[5]) - 1) * 1e6 ]) #val6
                #val[6].extend([ ( (float(row[7]) / vref[6]) - 1) * 1e6 ]) #val6
                #val[7].extend([ ( (float(row[8]) / vref[7]) - 1) * 1e6 ]) #val6
                uncerr[0].extend([0.2])
                uncerr[1].extend([0.62])
                #
                #val[8].extend([ ( (float(row[9]) / vref[8]) - 1) * 1e6 ]) #val6
                #val[9].extend([ ( (float(row[10]) / vref[9]) - 1) * 1e6 ]) #val6
                #val[10].extend([ ( (float(row[11]) / vref[10]) - 1) * 1e6 ]) #val6
                #val[11].extend([ ( (float(row[12]) / vref[11]) - 1) * 1e6 ]) #val6
                #
                #val[12].extend([ ( (float(row[13]) / vref[12]) - 1) * 1e6 ]) #val6
                #val[13].extend([ ( (float(row[14]) / vref[13]) - 1) * 1e6 ]) #val6
                #val[14].extend([ ( (float(row[15]) / vref[14]) - 1) * 1e6 ]) #val6
                #val[15].extend([ ( (float(row[16]) / vref[15]) - 1) * 1e6 ]) #val6
                
                ral[0].extend([  float(row[1]) ]) #val1
                ral[1].extend([  float(row[2]) ]) #val2
                ral[2].extend([  float(row[5]) ]) #val3
                ral[3].extend([  float(row[6]) ]) #val4
                ral[4].extend([  float(row[7]) ]) #val5
                #ral[5].extend([  float(row[6]) ]) #val6
                #ral[6].extend([  float(row[7]) ]) #val6
                #ral[7].extend([  float(row[8]) ]) #val6
                #ral[8].extend([  float(row[9]) ]) #val6
                #ral[9].extend([  float(row[10]) ]) #val6
                #ral[10].extend([ float(row[11]) ]) #val6
                #ral[11].extend([ float(row[12]) ]) #val6
                #ral[12].extend([ float(row[13]) ]) #val6
                #ral[13].extend([ float(row[14]) ]) #val6
                #ral[14].extend([ float(row[15]) ]) #val6
                #ral[15].extend([ float(row[16]) ]) #val6
                
                #val[24].extend([0.0]) #val8
            #corr.extend([(float(row[2]) + float(row[6]) ) / 2])
            #date.extend([row[0]])
    csvfile.close()
    
    #print amb
    tdelta = np.amax(temp) - np.amin(temp)
    reftemp = 23.00
    nominal = []
    median = []
   
    #print "Temperature min = %.3f , max = %.3f, delta = %.3f C" % (np.amin(temp), np.amax(temp), tdelta)
    #print "REF temp = %.3f C" % reftemp
        
    print "\033[2J"
    with open(fn_output, 'a') as logfile:
        logfile.write ("\r\rFile = %s\r" % fn_input)
    logfile.close()
    
    for zval in range (0,col_active):
        #nominal = [9.999957591e+00, 9.999963348e+00, 0, 0, 1.000001420e+01, 9.999958800e+00, 9.515830000e-01, 0, 0]
        nominal.append(np.mean(ral[zval][:30]))
        polyz.append(np.polyfit(temp, ral[zval], 2))
        c2.append(polyz[zval][0])
        c1.append(polyz[zval][1])
        gain.append(polyz[zval][2])
        tvalue.append(c2[zval] * reftemp*reftemp + c1[zval]*reftemp + gain[zval])
        alpha.append((c1[zval] + 2*reftemp*c2[zval]) / tvalue[zval]*1e6 )
        beta.append(c2[zval] / tvalue[zval]*1e6 )
        delta.append((tvalue[zval] - nominal[zval]) / nominal[zval] * 1e6)
        zerotctemp.append(-alpha[zval] / 2 / beta[zval] + reftemp)
        valzerotc.append(nominal[zval] + (zerotctemp[zval]) * c1[zval] + (zerotctemp[zval])*2*c2[zval])
        median.append(np.median(ral[zval]))
        
        if (gain[zval] != 0):
            print "\033[37;1m\r\nNominal       :", nominal[zval]
            print ("Median        : %.8f" % median[zval])
            print ("Column number : %d " % zval), polyz[zval], "Total samples = %d" % len(ral[zval])
            print ("c1            : %e" % c1[zval])
            print ("c2            : %e" % c2[zval])
            print ("g             : %e" % gain[zval])
            print ("Value T23     : %.6f" % (tvalue[zval]))
            print ("\033[36;1mAlpha T23     : %.4f ppm/K       " % (alpha[zval])),
            print ("Beta      : %.4f ppm/K/2                " % (beta[zval])),
            print ("Delta T23 : %.3f ppm         \033[39;1m" % (delta[zval]))
            print ("\033[35;1mZero TC       : %.3f C \033[39;0m" % (zerotctemp[zval])),
            print ("\033[32;1m          Delta ZeroTC : %.6f\033[39;0m" % (valzerotc[zval]))
            
            with open(fn_output, 'a') as logfile:
                logfile.write ("\rColumn number : %d\r" % zval), polyz[zval], "Total samples = %d" % len(ral[zval])
                logfile.write ("Nominal       : %.8f\r" % nominal[zval])
                logfile.write ("c1            : %e\r" % c1[zval])
                logfile.write ("c2            : %e\r" % c2[zval])
                logfile.write ("g             : %e\r" % gain[zval])
                logfile.write ("Value T23     : %.8f\r" % (tvalue[zval]))
                logfile.write ("Alpha T23     : %.4f ppm/K     \r" % (alpha[zval])),
                logfile.write ("Beta          : %.4f ppm/K/2   \r" % (beta[zval])),
                logfile.write ("Delta T23     : %.3f ppm       \r" % (delta[zval]))
                logfile.write ("Zero TC       : %.3f C \r" % (zerotctemp[zval])),
                logfile.write ("Delta ZeroTC  : %.8f   \r" % (valzerotc[zval]))
            logfile.close()
    w = 20
    h = 16
    fig,ax = plt.subplots()
    fig.set_size_inches(w,h)
    plt.tight_layout(rect=[0.03, 0.03, 0.96, 0.98], w_pad=0.4, h_pad=0.4)                                # Reduce margins around the graph
        
    plt.xlabel('Timescale', fontsize=18)
    plt.ylabel('Deviation from 10V, ppm', fontsize=18) #$\mu$V
    
    #new_x = dates.num2date(date)
    avgpower = 16
    
    val1aver = gaussian_filter1d(val[0], sigma=avgpower)
    val2aver = gaussian_filter1d(val[1], sigma=avgpower)
    val3aver = gaussian_filter1d(val[2], sigma=avgpower)
    val4aver = gaussian_filter1d(val[3], sigma=avgpower)
    val5aver = gaussian_filter1d(val[4], sigma=avgpower)
    #val6aver = gaussian_filter1d(val[5], sigma=avgpower)
    #val7aver = gaussian_filter1d(val[6], sigma=avgpower)
    #val8aver = gaussian_filter1d(val[7], sigma=avgpower)
    #
    #val9aver = gaussian_filter1d(val[8],   sigma=avgpower)
    #val10aver = gaussian_filter1d(val[9],  sigma=avgpower)
    #val11aver = gaussian_filter1d(val[10], sigma=avgpower)
    #val12aver = gaussian_filter1d(val[11], sigma=avgpower)
    #val13aver = gaussian_filter1d(val[12], sigma=avgpower)
    #val14aver = gaussian_filter1d(val[13], sigma=avgpower)
    #val15aver = gaussian_filter1d(val[14], sigma=avgpower)
    #val16aver = gaussian_filter1d(val[15], sigma=avgpower)
    
    temp2aver = gaussian_filter1d(temp, sigma=1)
    
    colors = ['#00aa00', '#005599', '#009999', '#551ac1', '#e51ac1', '#f58231', '#ff0000', '#f032e6', '#00a68c', '#664433', '#1f38e3', '#ff0000', '#ec18a0', '#dbb201', '#1c2b33', '#003355', '#fc007f']
    
    avglinestype = 'none'
    avglinestypen = 'none'
    
    plt.title('TheSteve snake 5 DMMs, 10VDC log from 732B', fontsize=20)
    ax.plot(date, val1aver, '-', label='CHA',  linewidth=1.0, color=colors[0], linestyle=avglinestype, marker='.', ms=2)
    ax.plot(date, val2aver, '-', label='CH2',  linewidth=1.0, color=colors[1], linestyle=avglinestype, marker='.', ms=2)
    ax.plot(date, val3aver, '-', label='CH3',  linewidth=1.0, color=colors[2], linestyle=avglinestype, marker='.', ms=2)
    ax.plot(date, val4aver, '-', label='CHB',  linewidth=1.0, color=colors[3], linestyle=avglinestype, marker='.', ms=2)
    ax.plot(date, val5aver, '-', label='CHC',  linewidth=1.0, color=colors[4], linestyle=avglinestype, marker='.', ms=2)
    #ax.plot(date, val6aver, '-', label='CH6',  linewidth=2.0, color=colors[5], linestyle=avglinestype, marker='.', ms=5)
    #ax.plot(date, val7aver, '-', label='CH7',  linewidth=2.0, color=colors[6], linestyle=avglinestype, marker='.', ms=5)
    #ax.plot(date, val8aver, '-', label='CH8',  linewidth=2.0, color=colors[7], linestyle=avglinestype, marker='.', ms=5)
    #ax.plot(date, val9aver, '-', label='CH1',  linewidth=3.0, color=colors[4], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val10aver, '-', label='CH2', linewidth=1.0, color=colors[4], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val11aver, '-', label='CH3', linewidth=3.0, color=colors[5], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val12aver, '-', label='CH4', linewidth=6.0, color=colors[6], linestyle=avglinestype)#, marker='.', ms=1)
    #ax.plot(date, val13aver, '-', label='CH5', linewidth=3.0, color=colors[7], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val14aver, '-', label='CH6', linewidth=1.0, color=colors[7], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val15aver, '-', label='CH7', linewidth=3.0, color=colors[8], linestyle=avglinestype, marker='.', ms=1)
    #ax.plot(date, val16aver, '-', label='CH8', linewidth=1.0, color=colors[8], linestyle=avglinestype, marker='.', ms=1)
    
    yerr = np.linspace(0.05, 0.2, 10)
    #plt.errorbar(date, val1aver, uncerr[0], errorevery = 50,  color=colors[0], uplims=True, lolims=True, linewidth=0.5, mew=0.2, label='both limits (default)')
    #plt.errorbar(date, val2aver, uncerr[0], errorevery = 40,  color=colors[3], uplims=True, lolims=True, linewidth=0.5, mew=0.2, label='both limits (default)')
    #plt.errorbar(date, val8aver, uncerr[1], errorevery = 70,  color=colors[7], uplims=True, lolims=True, linewidth=0.5, mew=0.2, label='both limits (default)')
    
    
    ax.plot(date, val[0], '-'  , label='CHA',  linewidth=0.8, alpha=0.2, color=colors[0] )#, linestyle='none', marker='.', ms=6)
    ax.plot(date, val[1], '-'  , label='CH2',  linewidth=0.8, alpha=0.2, color=colors[1] )#, linestyle='none', marker='.', ms=6)
    ax.plot(date, val[2], '-'  , label='CH3',  linewidth=0.8, alpha=0.2, color=colors[2] )#, linestyle='none', marker='.', ms=6)
    ax.plot(date, val[3], '-'  , label='CHB',  linewidth=0.8, alpha=0.2, color=colors[3] )#, linestyle='none', marker='.', ms=6)
    ax.plot(date, val[4], '-'  , label='CH4',  linewidth=0.8, alpha=0.2, color=colors[4] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[5], '-'  , label='CH6',  linewidth=1.0, alpha=0.3, color=colors[5] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[6], '-'  , label='CH7',  linewidth=1.0, alpha=0.3, color=colors[6] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[7], '-'  , label='CH8',  linewidth=1.0, alpha=0.3, color=colors[7] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[8], '-'  , label='CH1',  linewidth=1.0, alpha=0.3, color=colors[4] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[9], '-'  , label='CH2',  linewidth=1.0, alpha=0.3, color=colors[4] )#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[10], '-'  , label='CH3', linewidth=1.0, alpha=0.3, color=colors[5])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[11], '-'  , label='CH4', linewidth=1.0, alpha=0.3, color=colors[6])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[12], '-'  , label='CH5', linewidth=1.0, alpha=0.3, color=colors[7])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[13], '-'  , label='CH6', linewidth=1.0, alpha=0.3, color=colors[7])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[14], '-'  , label='CH7', linewidth=1.0, alpha=0.3, color=colors[8])#, linestyle='none', marker='.', ms=6)
    #ax.plot(date, val[15], '-'  , label='CH8', linewidth=1.0, alpha=0.3, color=colors[8])#, linestyle='none', marker='.', ms=6)
    
    
    #ax.plot(date, temp2aver, '-', label='TEMP', linewidth=3.0, color='#000000', linestyle='dotted', marker='.', ms=1)
    
    #start, end = ax.get_ylim()
    #startx, endx = ax.get_xlim()
    axis_y_step = 0.01
    #axis_x_step = 1
    ex = np.arange(1,91000)
    #ax.yaxis.set_ticks(np.arange(start, end)) # axis_y_step))
    ax.yaxis.set_ticks(np.arange(-4.0, 2, 0.1)) # axis_y_step))
    ax.set_ylim(auto=False)
    #ax.xaxis.set_ticks(np.arange(20.0, 26, 0.05)) # axis_y_step))
    #ax.xaxis.set_ticks(np.arange(0, 90000, 5000)) # axis_y_step))
    #ax.xaxis.set_ticks(np.arange(startx, endx)) #, axis_x_step))
    
    ax2 = ax.twinx()  # instantiate a second axes that shares the same x-axis
    ax2.set_ylabel(u'Temperature, °C', fontsize=18)  # we already handled the x-label with ax1
    ax.margins(x=0)
    ax2.margins(x=0)
    ax2.plot(date, temp, '-.', label='TEMP?', linewidth=1.0, color='#ff0000', linestyle='dashed', marker='.', ms=4)
    #ax2.plot(cnx, temp, '-.', label='TEMP?', linewidth=1.0, color='#000000', linestyle='dashed', marker='.', ms=4)
    ax2.tick_params(axis='y', labelcolor='#000000')
    legendlabel = ["3458[4]", "34461A", "34401A", "D1281", "3458[3]"]#, "3458-10", "3458-11", "2002-6"]#, "3458D vs 732B"]#, "K2002-6 vs 732B", "K2002-4", "8508A vs 732B", "D1281 vs SR104"]# "9.9815132 CH8", "7.1083426 CH9", "9.9231395 CH10", "7.1560244 CH11", "FX 792X", "7.1601550 CH13", "1.0000411 CH14", "7.1625227 CH15", "9.9966437 CH16", "Zero Ref"]
    legendlabel2 = [u"Ambient, °C"]
    ax.legend(legendlabel, loc=4, fancybox=True, framealpha=1, borderpad=1, markerscale=4)
    ax.grid(True, color='#cccccc')
    ax2.legend(legendlabel2, loc=2, fancybox=True, framealpha=1, borderpad=1, markerscale=4)
    ax2.yaxis.set_ticks(np.arange(18, 26, 0.1)) # axis_y_step))
    #ax.xaxis.set_major_formatter(mdates.DateFormatter("%b %d, %H:%M:%S"))
    #ax.xaxis.set_minor_formatter(mdates.DateFormatter("%b %d, %H:%M:%S"))

    vmed = []
    vmin = []
    vmax = []
    sdev = []
    
    tmin = np.amin(temp)
    tmax = np.amax(temp)
    temppp = tmax - 18
    tempco = []
    colormap = [0,1,2,3,4,5,6,7,8,9,5,6,7,7,8,8]
    
    for label in (ax.get_xticklabels() + ax.get_yticklabels()):
        #label.set_fontname('Arial')
        label.set_fontsize(13)
    
    for stats in range (0, col_active):
        vmed.extend([0])
        vmin.extend([0])
        vmax.extend([0])
        sdev.extend([0])
        tempco.extend([0])
        
    for stats in range (0, col_active):
    
        if (stats == 5):
            continue
    
        vmed[stats] = np.median(val[stats])
        vmin[stats] = np.amin(val[stats])# - vstep
        vmax[stats] = np.amax(val[stats])# + vstep
        sdev[stats] = np.std(val[stats]) # SDEV
        tempco[stats] = ((vmax[stats] - vmin[stats]) / temppp) 
        if ((stats % 2) == 0):
            bgclr = "#ffffff"
        else:
            bgclr = "#eeeeff"
        
        spacing_xleg = 0.1
        
        yspacing = 0.0175
        
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 1), 'Max[%d] : %.3f ppm' % (stats, vmax[stats]),
            horizontalalignment='left',
            verticalalignment='top', backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 2), 'Min[%d] : %.3f ppm' % (stats, vmin[stats]),
            horizontalalignment='left',
            verticalalignment='top', backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 3),'Median[%d] : %.3f ppm' % (stats, vmed[stats]),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 4),'Sdev[%d] : %.3f ppm' % (stats, sdev[stats]),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        #if (stats >6):
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 10.5), '%s' % (legendlabel[stats - 1]),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=12, color=colors[colormap[stats]])
        #else:
        #    plt.text(0.01 + (stats * spacing_xleg), (yspacing * 6.5), '%s' % (legendlabel[stats]),
        #        horizontalalignment='left',
        #        verticalalignment='top',backgroundcolor=bgclr,
        #        transform = ax.transAxes, size=12, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 9.5), 'R: %.7e' % (vref[stats]),
            horizontalalignment='left',
            verticalalignment='top', backgroundcolor=bgclr,
            transform = ax.transAxes, size=12, color=colors[colormap[stats]])
        
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 5), 'TCbox : %.3f ppm/K' % ( tempco[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 6), 'Alpha : %.3f ppm/K' % ( alpha[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 7), 'Beta : %.3f ppm/K2' % ( beta[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        plt.text(0.01 + (stats * spacing_xleg), (yspacing * 8), 'Zero TC : %.3f C' % ( zerotctemp[stats] ),
            horizontalalignment='left',
            verticalalignment='top',backgroundcolor=bgclr,
            transform = ax.transAxes, size=9, color=colors[colormap[stats]])
        
    fig.patch.set_facecolor('#eeeeff')
    smax = len(val[0])
    plt.text(0.2, -0.04,'xDevs.com | https://xdevs.com/article/teckit',
        horizontalalignment='center',
        verticalalignment='center',
        transform = ax.transAxes, size=12)
    plt.text(-0.01, -0.03,'Samples : %d' % smax,
        horizontalalignment='left',
        verticalalignment='top',
        transform = ax.transAxes, size=12)
    
    #plt.ylim(-2,2)
    plot_out_filename = "test_precal-scan_test_steve.png"
    fig.savefig(plot_out_filename, facecolor=fig.get_facecolor(), transparent=False)
    plt.show()